{*************************************************************************}
{                                                                         }
{  PROGRAM        : UNIPLOT                                               }
{  Version        : 1.6                                                  }
{  Sprache        : TURBO PASCAL 7.0                                      }
{  Aufgabe        : UNIPLOT simuliert einen PC ber IEEE488-Schnittstelle }
{                   als adressierbaren Plotter. Alle empfangenen Daten    }
{                   werden dabei in eine ASCII-Datei geschrieben.         }
{                   Der PC arbeitet hierbei als TALKER/LISTENER am        }
{                   IEEE488-Bus.                                          }
{                                                                         }
{  Beginn         : 03.09.1993                                            }
{  Autor          : Roland Kastner RSV-M                                  }
{  Letzte nderung: 05.10.1998                                            }
{                                                                         }
{  (C) Rohde & Schwarz Vertriebs GmbH, Support Center RSV-ZS              }
{                                                                         }
{*************************************************************************}
{*************************************************************************}
{                                                                         }
{ Copyright by ROHDE & SCHWARZ, Munich, Federal Republic of               }
{ Germany. All rights reserved.                                           }
{                                                                         }
{ This software is furnished under a license and may be used              }
{ and copied only in accordance with the terms of such license            }
{ and with the inclusion of the above Copyright Notice. This              }
{ software or any other copies thereof may not be provided or             }
{ otherwise made available to any other person. No title to and           }
{ ownership of the software is hereby transferred.                        }
{                                                                         }
{ The information in this software is subject to change without           }
{ notice and should not be construed as a commitment by ROHDE &           }
{ SCHWARZ.                                                                }
{                                                                         }
{ ROHDE & SCHWARZ assumes no responsibility for the use or re-            }
{ liability of its software on equipment which is not supplied            }
{ by ROHDE & SCHWARZ.                                                     }
{                                                                         }
{*************************************************************************}
{*************************************************************************}
{ Logbuch der Versionen:                                                  }
{ 18.02.1996 V1.5   Steuerung ber Kommandozeilen Parameter               }
{ 05.10,1998 V1.6   Wg. CRT Problem bei schnellen Pentium mit gepatcher   }
{                   TURBO.TPL neu compiliert                              }
{                                                                         }
{*************************************************************************}
program plotter;

{$M 5120, 0, 5120}

uses tpdecl,dos,crt;

CONST
  Default_File = 'DATA.HGL';
  Default_adr  = 5;

var
  bd,v,mask,adr,tmo,z, xPos, YPos : integer;
  buffer,ch : char;
  DataFile1 : string;
  DataFile  : string;
  Datei     : text;

type
  CharSet = set of Char;

{-------------------------------------------------------------------------}
{ Name: StrToInt  ; Function   Typ: Integer                               }
{                                                                         }
{ Aufgabe: Wandelt Zahlen innerhalb  von Strings in Longinteger-Werte     }
{          z.B. 'A DBM 113.4E-1' -> 113 als Longinteger-Wert              }
{                                                                         }
{ IN  : Str       Typ:  String    Bem: Zu wandelnder String               }
{-------------------------------------------------------------------------}
function StrToInt (Str: String): Longint;
var
  I,Err: Integer;
  HStr : String;
  HChr : Char;
  R    : Real;

Const
  Token : CharSet = ['0'..'9','E','e','-','.'];

begin
  HStr:= '';
  For I:= 1 to ord(Str[0]) do
  begin
    HChr:= Str[I];
    if HChr IN Token then HStr:= HStr+HChr;
  end;
  val(HStr, R, Err);
  StrToInt:= trunc(R);
end;
{*************************************************************************}

{-------------------------------------------------------------------------}
{ Name: StUpCase  ; Function             Typ: String                      }
{                                                                         }
{ Aufgabe: Wandelt String komplett in Grobuchstaben                      }
{                                                                         }
{ IN  : St; Zu wandelnder String           Typ:  String                   }
{-------------------------------------------------------------------------}
function StUpCase(St: String): String;
var
  i : byte;

begin
  for i:= 1 to ord(St[0]) do
  begin
    case St[i] of
      '': St[i]:= '';
      '': St[i]:= '';
      '': St[i]:= '';
    else
      St[i]:= UpCase(St[i]);
    end;
  end;
  StUpCase:= St;
end;
{-------------------------------------------------------------------------}

{-------------------------------------------------------------------------}
procedure HelpText;
begin
  Writeln;
  Writeln('  UNIPLOT V1.60, Copyright 1992, 98 by ROHDE & SCHWARZ Vertrieb, NL. Muenchen');
  Writeln('  SAVE HPGL PLOTTER-DATA TO ASCII FILE');
  Writeln;
  Writeln('  UNIPLOT [/F:Filename] [/A:IEEE adress]');
  Writeln('    /HELP  This help');
  Writeln('    /F:    Name of HPGL-file; Default "DATA.HGL" ');
  Writeln('    /A:nn  GPIB-Adress of plotter/PC (1..30); Default = 5');
  Writeln;
end;
{-------------------------------------------------------------------------}

{-------------------------------------------------------------------------}
procedure status;
var  zw: integer;
     anz: string[20];
begin
 anz:= '';
 zw := ibsta;
 if (zw and ERR)  <>0 then anz:='ERR ';
 if (zw and TIMO) <>0 then anz:=anz+'TIMO ';
 if (zw and UEND) <>0 then anz:=anz+'END ';
 if (zw and SRQI) <>0 then anz:=anz+'SRQI ';
 if (zw and RQS)  <>0 then anz:=anz+'RQS ';
 if (zw and CMPL) <>0 then anz:=anz+'CMPL ';
 if (zw and LOK)  <>0 then anz:=anz+'LOK ';
 if (zw and REM)  <>0 then anz:=anz+'REM ';
 if (zw and CIC)  <>0 then anz:=anz+'CIC ';
 if (zw and ATN)  <>0 then anz:=anz+'ATN ';
 if (zw and TACS) <>0 then anz:=anz+'TACS ';
 if (zw and LACS) <>0 then anz:=anz+'LACS ';
 if (zw and DTAS) <>0 then anz:=anz+'DTAS ';
 if (zw and DCAS) <>0 then anz:=anz+'DCAS ';
 writeln;write('Status word(s): '); writeln(anz);
 writeln;
end;
{-------------------------------------------------------------------------}

{-------------------------------------------------------------------------}
procedure finderror; { Grundstzliches Problem mit dem GPIB-Interface }
begin
 writeln;
 writeln('ERROR: GPIB-PC board not found or GPIB.COM unloaded !');
 writeln('       Programm terminated');
 writeln;
 halt;
end;
{-------------------------------------------------------------------------}

{-------------------------------------------------------------------------}
procedure error;

var  zw: integer;

begin
 if ibsta < 0 then
 begin
   zw:=iberr;
   write('Error code ');
   if (zw and EDVR)<>0 then writeln('EDVR: DOS error.');
   if (zw and ECIC)<>0 then writeln('ECIC: Function requires GPIB-PC to be CIC.');
   if (zw and ENOL)<>0 then writeln('ENOL: No listener on write function.');
   if (zw and EADR)<>0 then writeln('EADR: GPIB-PC not addressed correctly.');
   if (zw and EARG)<>0 then writeln('EARG: Invalid argument to function call.');
   if (zw and ESAC)<>0 then writeln('ESAC: GPIB-PC not system controller as required.');
   if (zw and EABO)<>0 then writeln('EABO: I/O operation aborted.');
   if (zw and ENEB)<>0 then writeln('ENEB: Non-existent GPIB-PC board.');
   if (zw and EOIP)<>0 then writeln('EOIP: I/O started before previous operation completed.');
   if (zw and ECAP)<>0 then writeln('ECAP: No capability for operation.');
   if (zw and EFSO)<>0 then writeln('EFSO: File system error.');
   if (zw and EBUS)<>0 then writeln('EBUS: Command error during device call.');
   if (zw and ESTB)<>0 then writeln('ESTB: Serial status byte lost.');
   if (zw and ESRQ)<>0 then writeln('ESRQ: SRQ stuck in on position.');
 end;
 halt;
end;

{-------------------------------------------------------------------------}
procedure Fertig;
begin
  writeln;
  if UpCase(Ch) <> 'Q' then
  Begin
    writeln('Ready !');
    close(Datei);
    v:= 1;
    ibrsc(bd,v);                         { Kontrolle erteilen }
    if (ibsta < 0) then error;
    ibsic(bd);                           { Reset Karte }
  End;
end;

{**********************************************************************}
{                  H A U P T P R O G R A M M                           }
{**********************************************************************}
Var
  I :  Integer;
  Hstr: String;

begin
  Clrscr;
  If Paramcount > 0 then                { Kommandozeilen-parameter}
  begin
    For I:= 1 to ParamCount do
    begin
      HStr:= StUpCase(ParamStr(I));
      if (HStr = '/HELP') OR (HStr = '/H') OR (HStr ='') OR
         (HStr = '?') OR (HStr = '/?') then { Hilfe-Anzeige der Kommandozeile }
      begin
        HelpText;
        Halt (0);
      end;

      if copy(HStr,1,3) = '/F:' then      { Filename }
      begin
        DataFile:= copy(HStr,4, length(HStr) - 3);
      end;

      if copy(HStr,1,3) = '/A:' then      { IEC-Bus Adresse }
      begin
        Adr:= StrToInt(copy(HStr,4, length(HStr) - 3));
      end;
    end;
  end;
  if (DataFile = '') OR (Length(DataFile) > 12) then
     DataFile:= Default_File;
  If (Adr < 1) OR (Adr > 30) then Adr:= Default_Adr;
  HelpText;

  bdname:='gpib0  ';
  bd := ibfind (bdname);
  if (bd < 0) then finderror;

  writeln('Press any key to continue or Q to Quit');

  Ch:= ReadKey;
  if (Ch = 'Q') OR (Ch = 'q') then fertig;

  assign(Datei,DataFile);
  rewrite(Datei);

 { bdname:='gpib0  ';
  bd := ibfind (bdname);
  if (bd < 0) then finderror;
  }
  v:=0;
  ibeos(bd,v);                         { EOS-Mode aus }
  if (ibsta < 0) then error;

  ibpad(bd,adr);                       { Setzen der IEC-BUS-Adresse }
  if (ibsta < 0) then error;

  ibtmo(bd,T30s);                      { Setzen der Timeout-Zeit }
  if (ibsta < 0) then error;

  v:=0;
  ibrsc(bd,v);                         { Kontrolle abgeben }
  if (ibsta < 0) then error;

  writeln('Waiting to be addressed...');

  mask:=LACS+TIMO;
  ibwait(bd,mask);
  if (ibsta < 0) then error;

  if (ibsta and TIMO) <> 0 then
  begin
    writeln('Timelimit exceeded!');
    fertig;
  end;

  writeln('Addressed as listner ...');
  z:=0;
  writeln;
  xPos:=WhereX; yPos:=WhereY;
  GotoXY(xPos,Ypos-1);
  write('Bytes received: ');
  xPos:=WhereX; yPos:=WhereY;

  ibtmo(bd,T3s);                       { Setzen der Timeout-Zeit }
  if (ibsta < 0) then error;

while (ibsta and TIMO) = 0 do
begin
   ibrd(bd, buffer,1);
   if (ibsta and TIMO)<>0 then exit;
   inc(z);
   write(datei,buffer);
   GotoXY(xPos,Ypos);
   write(z);
end;

Fertig;
end.